package com.ruoyi.ems.service;

import com.ruoyi.ems.domain.Area;

import java.util.List;

/**
 * 区域对象Service接口
 *
 * @author ruoyi
 * @date 2024-11-01
 */
public interface IAreaService {
    /**
     * 查询区域对象
     *
     * @param id 区域对象主键
     * @return 区域对象
     */
    Area selectAreaById(Long id);

    /**
     * 查询区域对象
     *
     * @param areaCode 区域对象编码
     * @return 区域对象
     */
    Area selectAreaByCode(String areaCode);

    /**
     * 查询区域位置树
     *
     * @param recursion  是否递归
     * @param parentCode 上级代码
     * @return 区域位置树
     */
    List<Area> selectAreaTree(String parentCode, boolean recursion);

    /**
     * 查询区域对象列表
     *
     * @param area 区域对象
     * @return 区域对象集合
     */
    List<Area> selectArea(Area area);

    /**
     * 查询区域对象列表
     *
     * @param area 区域对象
     * @return 区域对象集合
     */
    List<Area> selectAreaDetail(Area area);

    /**
     * 查询标签绑定的区域
     * @param tagCode 标签
     * @return 区域对象集合
     */
    List<Area> selectAreaByTag(String tagCode);

    /**
     * 新增区域对象
     *
     * @param area 区域对象
     * @return 结果
     */
    int insertArea(Area area);

    /**
     * 修改区域对象
     *
     * @param area 区域对象
     * @return 结果
     */
    int updateArea(Area area);

    /**
     * 批量删除区域对象
     *
     * @param ids 需要删除的区域对象主键集合
     * @return 结果
     */
    int deleteAreaByIds(Long[] ids);

    /**
     * 删除区域对象信息
     *
     * @param id 区域对象主键
     * @return 结果
     */
    int deleteAreaById(Long id);
}
